@@ -6,6 +6,8 @@ use cargo::util::important_paths::find_root_manifest_for_cwd;
 struct Options {
     flag_manifest_path: Option<String>,
     flag_verbose: bool,
+    flag_quiet: bool,
+    flag_color: Option<String>,
 }
 
 pub const USAGE: &'static str = "
@@ -15,9 +17,11 @@ Usage:
     cargo fetch [options]
 
 Options:
-    -h, --help              Print this message
-    --manifest-path PATH    Path to the manifest to fetch dependencies for
-    -v, --verbose           Use verbose output
+    -h, --help               Print this message
+    --manifest-path PATH     Path to the manifest to fetch dependencies for
+    -v, --verbose            Use verbose output
+    -q, --quiet              No output printed to stdout
+    --color WHEN             Coloring: auto, always, never
 
 If a lockfile is available, this command will ensure that all of the git
 dependencies and/or registries dependencies are downloaded and locally
@@ -30,7 +34,8 @@ all updated.
 ";
 
 pub fn execute(options: Options, config: &Config) -> CliResult<Option<()>> {
-    config.shell().set_verbose(options.flag_verbose);
+    try!(config.shell().set_verbosity(options.flag_verbose, options.flag_quiet));
+    try!(config.shell().set_color_config(options.flag_color.as_ref().map(|s| &s[..])));
     let root = try!(find_root_manifest_for_cwd(options.flag_manifest_path));
     try!(ops::fetch(&root, config).map_err(|e| {
         CliError::from_boxed(e, 101)
@@ -38,4 +43,3 @@ pub fn execute(options: Options, config: &Config) -> CliResult<Option<()>> {
     Ok(None)
 }
 
-
