@@ -8,6 +8,8 @@ struct Options {
     flag_token: Option<String>,
     flag_manifest_path: Option<String>,
     flag_verbose: bool,
+    flag_quiet: bool,
+    flag_color: Option<String>,
     flag_no_verify: bool,
 }
 
@@ -18,17 +20,20 @@ Usage:
     cargo publish [options]
 
 Options:
-    -h, --help              Print this message
-    --host HOST             Host to upload the package to
-    --token TOKEN           Token to use when uploading
-    --no-verify             Don't verify package tarball before publish
-    --manifest-path PATH    Path to the manifest to compile
-    -v, --verbose           Use verbose output
+    -h, --help               Print this message
+    --host HOST              Host to upload the package to
+    --token TOKEN            Token to use when uploading
+    --no-verify              Don't verify package tarball before publish
+    --manifest-path PATH     Path to the manifest to compile
+    -v, --verbose            Use verbose output
+    -q, --quiet              No output printed to stdout
+    --color WHEN             Coloring: auto, always, never
 
 ";
 
 pub fn execute(options: Options, config: &Config) -> CliResult<Option<()>> {
-    config.shell().set_verbose(options.flag_verbose);
+    try!(config.shell().set_verbosity(options.flag_verbose, options.flag_quiet));
+    try!(config.shell().set_color_config(options.flag_color.as_ref().map(|s| &s[..])));
     let Options {
         flag_token: token,
         flag_host: host,
