@@ -4,15 +4,16 @@ use std::slice;
 use std::path::{Path, PathBuf};
 use semver::Version;
 
-use core::{Dependency, Manifest, PackageId, Registry, Target, Summary, Metadata};
+use core::{Dependency, Manifest, PackageId, SourceId, Registry, Target, Summary, Metadata};
+use ops;
 use core::dependency::SerializedDependency;
-use util::{CargoResult, graph};
+use util::{CargoResult, graph, Config};
 use rustc_serialize::{Encoder,Encodable};
 use core::source::Source;
 
-/// Informations about a package that is available somewhere in the file system.
+/// Information about a package that is available somewhere in the file system.
 ///
-/// A package is a `Cargo.toml` file, plus all the files that are part of it.
+/// A package is a `Cargo.toml` file plus all the files that are part of it.
 // TODO: Is manifest_path a relic?
 #[derive(Clone, Debug)]
 pub struct Package {
@@ -58,6 +59,14 @@ impl Package {
         }
     }
 
+    pub fn for_path(manifest_path: &Path, config: &Config) -> CargoResult<Package> {
+        let path = manifest_path.parent().unwrap();
+        let source_id = try!(SourceId::for_path(path));
+        let (pkg, _) = try!(ops::read_package(&manifest_path, &source_id,
+                                              config));
+        Ok(pkg)
+    }
+
     pub fn dependencies(&self) -> &[Dependency] { self.manifest.dependencies() }
     pub fn manifest(&self) -> &Manifest { &self.manifest }
     pub fn manifest_path(&self) -> &Path { &self.manifest_path }
