@@ -28,15 +28,14 @@ use std::path::{Path, PathBuf};
 use std::sync::Arc;
 
 use core::registry::PackageRegistry;
-use core::{Source, SourceId, PackageSet, Package, Target, PackageId};
-use core::{Profile, TargetKind};
+use core::{Source, SourceId, PackageSet, Package, Target};
+use core::{Profile, TargetKind, Profiles};
 use core::resolver::Method;
 use ops::{self, BuildOutput, ExecEngine};
-use sources::{PathSource};
 use util::config::{ConfigValue, Config};
 use util::{CargoResult, internal, human, ChainError, profile};
 
-/// Contains informations about how a package should be compiled.
+/// Contains information about how a package should be compiled.
 pub struct CompileOptions<'a> {
     pub config: &'a Config,
     /// Number of concurrent jobs to use.
@@ -48,7 +47,7 @@ pub struct CompileOptions<'a> {
     /// Flag if the default feature should be built for the root package
     pub no_default_features: bool,
     /// Root package to build (if None it's the current one)
-    pub spec: Option<&'a str>,
+    pub spec: &'a [String],
     /// Filter to apply to the root package to select which targets will be
     /// built.
     pub filter: CompileFilter<'a>,
@@ -87,21 +86,17 @@ pub fn compile<'a>(manifest_path: &Path,
                    -> CargoResult<ops::Compilation<'a>> {
     debug!("compile; manifest-path={}", manifest_path.display());
 
-    let mut source = try!(PathSource::for_path(manifest_path.parent().unwrap(),
-                                               options.config));
-    try!(source.update());
-
-    // TODO: Move this into PathSource
-    let package = try!(source.root_package());
+    let package = try!(Package::for_path(manifest_path, options.config));
     debug!("loaded package; package={}", package);
 
     for key in package.manifest().warnings().iter() {
         try!(options.config.shell().warn(key))
     }
-    compile_pkg(&package, Some(Box::new(source)), options)
+    compile_pkg(&package, None, options)
 }
 
-pub fn compile_pkg<'a>(package: &Package,
+#[allow(deprecated)] // connect => join in 1.3
+pub fn compile_pkg<'a>(root_package: &Package,
                        source: Option<Box<Source + 'a>>,
                        options: &CompileOptions<'a>)
                        -> CargoResult<ops::Compilation<'a>> {
@@ -115,7 +110,7 @@ pub fn compile_pkg<'a>(package: &Package,
         s.split(' ')
     }).map(|s| s.to_string()).collect::<Vec<String>>();
 
-    if spec.is_some() && (no_default_features || features.len() > 0) {
+    if spec.len() > 0 && (no_default_features || features.len() > 0) {
         return Err(human("features cannot be modified when the main package \
                           is not being built"))
     }
@@ -123,21 +118,18 @@ pub fn compile_pkg<'a>(package: &Package,
         return Err(human("jobs must be at least 1"))
     }
 
-    let override_ids = try!(source_ids_from_config(config, package.root()));
+    let override_ids = try!(source_ids_from_config(options.config, root_package.root()));
 
     let (packages, resolve_with_overrides, sources) = {
-        let rustc_host = config.rustc_host().to_string();
-        let mut registry = PackageRegistry::new(config);
+        let mut registry = PackageRegistry::new(options.config);
+
         if let Some(source) = source {
-            registry.preload(package.package_id().source_id(), source);
-        } else {
-            try!(registry.add_sources(&[package.package_id().source_id()
-                                               .clone()]));
+            registry.add_preloaded(root_package.package_id().source_id(), source);
         }
 
-        // First, resolve the package's *listed* dependencies, as well as
+        // First, resolve the root_package's *listed* dependencies, as well as
         // downloading and updating all remotes and such.
-        let resolve = try!(ops::resolve_pkg(&mut registry, package));
+        let resolve = try!(ops::resolve_pkg(&mut registry, root_package));
 
         // Second, resolve with precisely what we're doing. Filter out
         // transitive dependencies if necessary, specify features, handle
@@ -146,55 +138,84 @@ pub fn compile_pkg<'a>(package: &Package,
 
         try!(registry.add_overrides(override_ids));
 
-        let platform = target.as_ref().map(|e| &e[..]).or(Some(&rustc_host[..]));
-
         let method = Method::Required{
             dev_deps: true, // TODO: remove this option?
             features: &features,
             uses_default_features: !no_default_features,
-            target_platform: platform};
+        };
 
         let resolved_with_overrides =
-                try!(ops::resolve_with_previous(&mut registry, package, method,
-                                                Some(&resolve), None));
+                try!(ops::resolve_with_previous(&mut registry, root_package,
+                                                method, Some(&resolve), None));
 
-        let req: Vec<PackageId> = resolved_with_overrides.iter().map(|r| {
-            r.clone()
-        }).collect();
-        let packages = try!(registry.get(&req).chain_error(|| {
-            human("Unable to get packages from source")
-        }));
+        let packages = try!(ops::get_resolved_packages(&resolved_with_overrides,
+                                                       &mut registry));
 
         (packages, resolved_with_overrides, registry.move_sources())
     };
 
-    let pkgid = match spec {
-        Some(spec) => try!(resolve_with_overrides.query(spec)),
-        None => package.package_id(),
-    };
-    let to_build = packages.iter().find(|p| p.package_id() == pkgid).unwrap();
-    let targets = try!(generate_targets(to_build, mode, filter, release));
-
-    let target_with_args = match *target_rustc_args {
-        Some(args) if targets.len() == 1 => {
-            let (target, profile) = targets[0];
-            let mut profile = profile.clone();
-            profile.rustc_args = Some(args.to_vec());
-            Some((target, profile))
-        }
-        Some(_) => {
-            return Err(human("extra arguments to `rustc` can only be passed to \
-                              one target, consider filtering\nthe package by \
-                              passing e.g. `--lib` or `--bin NAME` to specify \
-                              a single target"))
-        }
-        None => None,
+    let mut invalid_spec = vec![];
+    let pkgids = if spec.len() > 0 {
+        spec.iter().filter_map(|p| {
+            match resolve_with_overrides.query(&p) {
+                Ok(p) => Some(p),
+                Err(..) => { invalid_spec.push(p.to_string()); None }
+            }
+        }).collect::<Vec<_>>()
+    } else {
+        vec![root_package.package_id()]
     };
 
-    let targets = target_with_args.as_ref().map(|&(t, ref p)| vec![(t, p)])
-                                           .unwrap_or(targets);
+    if spec.len() > 0 && invalid_spec.len() > 0 {
+        return Err(human(format!("could not find package matching spec `{}`",
+                                 invalid_spec.connect(", "))));
+    }
 
-    let ret = {
+    let to_builds = packages.iter().filter(|p| pkgids.contains(&p.package_id()))
+                            .collect::<Vec<_>>();
+
+    let mut general_targets = Vec::new();
+    let mut package_targets = Vec::new();
+
+    let profiles = root_package.manifest().profiles();
+    match *target_rustc_args {
+        Some(args) => {
+            if to_builds.len() == 1 {
+                let targets = try!(generate_targets(to_builds[0], profiles,
+                                                    mode, filter, release));
+                if targets.len() == 1 {
+                    let (target, profile) = targets[0];
+                    let mut profile = profile.clone();
+                    profile.rustc_args = Some(args.to_vec());
+                    general_targets.push((target, profile));
+                } else {
+                    return Err(human("extra arguments to `rustc` can only be \
+                                      passed to one target, consider \
+                                      filtering\nthe package by passing e.g. \
+                                      `--lib` or `--bin NAME` to specify \
+                                      a single target"))
+
+                }
+            } else {
+                panic!("`rustc` should not accept multiple `-p` flags")
+            }
+        }
+        None => {
+            for &to_build in to_builds.iter() {
+                let targets = try!(generate_targets(to_build, profiles, mode,
+                                                    filter, release));
+                package_targets.push((to_build, targets));
+            }
+        }
+    };
+
+    for &(target, ref profile) in &general_targets {
+        for &to_build in to_builds.iter() {
+            package_targets.push((to_build, vec![(target, profile)]));
+        }
+    }
+
+    let mut ret = {
         let _p = profile::start("compiling");
         let mut build_config = try!(scrape_build_config(config, jobs, target));
         build_config.exec_engine = exec_engine.clone();
@@ -203,15 +224,18 @@ pub fn compile_pkg<'a>(package: &Package,
             build_config.doc_all = deps;
         }
 
-        try!(ops::compile_targets(&targets, to_build,
+        try!(ops::compile_targets(&package_targets,
                                   &PackageSet::new(&packages),
                                   &resolve_with_overrides,
                                   &sources,
                                   config,
                                   build_config,
-                                  to_build.manifest().profiles()))
+                                  root_package.manifest().profiles(),
+                                  ))
     };
 
+    ret.to_doc_test = to_builds.iter().map(|&p| p.clone()).collect();
+
     return Ok(ret);
 }
 
@@ -253,11 +277,11 @@ impl<'a> CompileFilter<'a> {
 /// Given the configuration for a build, this function will generate all
 /// target/profile combinations needed to be built.
 fn generate_targets<'a>(pkg: &'a Package,
+                        profiles: &'a Profiles,
                         mode: CompileMode,
                         filter: &CompileFilter,
                         release: bool)
                         -> CargoResult<Vec<(&'a Target, &'a Profile)>> {
-    let profiles = pkg.manifest().profiles();
     let build = if release {&profiles.release} else {&profiles.dev};
     let test = if release {&profiles.bench} else {&profiles.test};
     let profile = match mode {
@@ -397,7 +421,7 @@ fn scrape_build_config(config: &Config,
         requested_target: target.clone(),
         ..Default::default()
     };
-    base.host = try!(scrape_target_config(config, config.rustc_host()));
+    base.host = try!(scrape_target_config(config, &config.rustc_info().host));
     base.target = match target.as_ref() {
         Some(triple) => try!(scrape_target_config(config, &triple)),
         None => base.host.clone(),
@@ -409,12 +433,9 @@ fn scrape_target_config(config: &Config, triple: &str)
                         -> CargoResult<ops::TargetConfig> {
 
     let key = format!("target.{}", triple);
-    let ar = try!(config.get_string(&format!("{}.ar", key)));
-    let linker = try!(config.get_string(&format!("{}.linker", key)));
-
     let mut ret = ops::TargetConfig {
-        ar: ar.map(|p| p.0),
-        linker: linker.map(|p| p.0),
+        ar: try!(config.get_path(&format!("{}.ar", key))),
+        linker: try!(config.get_path(&format!("{}.linker", key))),
         overrides: HashMap::new(),
     };
     let table = match try!(config.get_table(&key)) {
