@@ -1,7 +1,6 @@
 use std::env;
-use std::fs::{self, File};
+use std::fs;
 use std::io::prelude::*;
-use std::io;
 use std::path::Path;
 
 use rustc_serialize::{Decodable, Decoder};
@@ -11,7 +10,7 @@ use git2::Config as GitConfig;
 use term::color::BLACK;
 
 use util::{GitRepo, HgRepo, CargoResult, human, ChainError, internal};
-use util::Config;
+use util::{Config, paths};
 
 use toml;
 
@@ -102,10 +101,6 @@ fn existing_vcs_repo(path: &Path) -> bool {
     GitRepo::discover(path).is_ok() || HgRepo::discover(path).is_ok()
 }
 
-fn file(p: &Path, contents: &[u8]) -> io::Result<()> {
-    try!(File::create(p)).write_all(contents)
-}
-
 fn mk(config: &Config, path: &Path, name: &str,
       opts: &NewOptions) -> CargoResult<()> {
     let cfg = try!(global_config(config));
@@ -118,18 +113,18 @@ fn mk(config: &Config, path: &Path, name: &str,
     let vcs = match (opts.version_control, cfg.version_control, in_existing_vcs_repo) {
         (None, None, false) => VersionControl::Git,
         (None, Some(option), false) => option,
-        (Some(option), _, false) => option,
+        (Some(option), _, _) => option,
         (_, _, true) => VersionControl::NoVcs,
     };
 
     match vcs {
         VersionControl::Git => {
             try!(GitRepo::init(path));
-            try!(file(&path.join(".gitignore"), ignore.as_bytes()));
+            try!(paths::write(&path.join(".gitignore"), ignore.as_bytes()));
         },
         VersionControl::Hg => {
             try!(HgRepo::init(path));
-            try!(file(&path.join(".hgignore"), ignore.as_bytes()));
+            try!(paths::write(&path.join(".hgignore"), ignore.as_bytes()));
         },
         VersionControl::NoVcs => {
             try!(fs::create_dir(path));
@@ -147,7 +142,7 @@ fn mk(config: &Config, path: &Path, name: &str,
         (None, None, name, None) => name,
     };
 
-    try!(file(&path.join("Cargo.toml"), format!(
+    try!(paths::write(&path.join("Cargo.toml"), format!(
 r#"[package]
 name = "{}"
 version = "0.1.0"
@@ -157,13 +152,13 @@ authors = [{}]
     try!(fs::create_dir(&path.join("src")));
 
     if opts.bin {
-        try!(file(&path.join("src/main.rs"), b"\
+        try!(paths::write(&path.join("src/main.rs"), b"\
 fn main() {
     println!(\"Hello, world!\");
 }
 "));
     } else {
-        try!(file(&path.join("src/lib.rs"), b"\
+        try!(paths::write(&path.join("src/lib.rs"), b"\
 #[test]
 fn it_works() {
 }
@@ -188,7 +183,8 @@ fn discover_author() -> CargoResult<(String, Option<String>)> {
                                       user, please set ${}", username_var)))
         }
     };
-    let email = git_config.and_then(|g| g.get_string("user.email").ok());
+    let email = git_config.and_then(|g| g.get_string("user.email").ok())
+                          .or_else(|| env::var("EMAIL").ok());
 
     let name = name.trim().to_string();
     let email = email.map(|s| s.trim().to_string());
