@@ -1,17 +1,13 @@
 use std::path::Path;
 
 use ops;
-use core::{Source, PackageIdSpec};
-use sources::{PathSource};
+use core::{PackageIdSpec, Package};
 use util::{CargoResult, human, Config};
 
 pub fn pkgid(manifest_path: &Path,
              spec: Option<&str>,
              config: &Config) -> CargoResult<PackageIdSpec> {
-    let mut source = try!(PathSource::for_path(&manifest_path.parent().unwrap(),
-                                               config));
-    try!(source.update());
-    let package = try!(source.root_package());
+    let package = try!(Package::for_path(manifest_path, config));
 
     let lockfile = package.root().join("Cargo.lock");
     let source_id = package.package_id().source_id();
@@ -21,7 +17,7 @@ pub fn pkgid(manifest_path: &Path,
     };
 
     let pkgid = match spec {
-        Some(spec) => try!(resolve.query(spec)),
+        Some(spec) => try!(PackageIdSpec::query_str(spec, resolve.iter())),
         None => package.package_id(),
     };
     Ok(PackageIdSpec::from_package_id(pkgid))
