@@ -12,8 +12,7 @@ use std::path::{Path, PathBuf};
 use rustc_serialize::{Encodable,Encoder};
 use toml;
 use core::{MultiShell, Package};
-use ops;
-use util::{CargoResult, ChainError, internal, human};
+use util::{CargoResult, ChainError, Rustc, internal, human, paths};
 
 use util::toml as cargo_toml;
 
@@ -22,15 +21,13 @@ use self::ConfigValue as CV;
 pub struct Config {
     home_path: PathBuf,
     shell: RefCell<MultiShell>,
-    rustc_version: String,
-    /// The current host and default target of rustc
-    rustc_host: String,
+    rustc_info: Rustc,
     values: RefCell<HashMap<String, ConfigValue>>,
     values_loaded: Cell<bool>,
     cwd: PathBuf,
     rustc: PathBuf,
     rustdoc: PathBuf,
-    target_dir: Option<PathBuf>,
+    target_dir: RefCell<Option<PathBuf>>,
 }
 
 impl Config {
@@ -45,14 +42,13 @@ impl Config {
                       This probably means that $HOME was not set.")
             })),
             shell: RefCell::new(shell),
-            rustc_version: String::new(),
-            rustc_host: String::new(),
+            rustc_info: Rustc::blank(),
             cwd: cwd,
             values: RefCell::new(HashMap::new()),
             values_loaded: Cell::new(false),
             rustc: PathBuf::from("rustc"),
             rustdoc: PathBuf::from("rustdoc"),
-            target_dir: None,
+            target_dir: RefCell::new(None),
         };
 
         try!(cfg.scrape_tool_config());
@@ -92,11 +88,7 @@ impl Config {
 
     pub fn rustdoc(&self) -> &Path { &self.rustdoc }
 
-    /// Return the output of `rustc -v verbose`
-    pub fn rustc_version(&self) -> &str { &self.rustc_version }
-
-    /// Return the host platform and default target of rustc
-    pub fn rustc_host(&self) -> &str { &self.rustc_host }
+    pub fn rustc_info(&self) -> &Rustc { &self.rustc_info }
 
     pub fn values(&self) -> CargoResult<Ref<HashMap<String, ConfigValue>>> {
         if !self.values_loaded.get() {
@@ -109,11 +101,15 @@ impl Config {
     pub fn cwd(&self) -> &Path { &self.cwd }
 
     pub fn target_dir(&self, pkg: &Package) -> PathBuf {
-        self.target_dir.clone().unwrap_or_else(|| {
+        self.target_dir.borrow().clone().unwrap_or_else(|| {
             pkg.root().join("target")
         })
     }
 
+    pub fn set_target_dir(&self, path: &Path) {
+        *self.target_dir.borrow_mut() = Some(path.to_owned());
+    }
+
     pub fn get(&self, key: &str) -> CargoResult<Option<ConfigValue>> {
         let vals = try!(self.values());
         let mut parts = key.split('.').enumerate();
@@ -154,6 +150,22 @@ impl Config {
         }
     }
 
+    pub fn get_path(&self, key: &str) -> CargoResult<Option<PathBuf>> {
+        if let Some((specified_path, path_to_config)) = try!(self.get_string(&key)) {
+            if specified_path.contains("/") || (cfg!(windows) && specified_path.contains("\\")) {
+                // An absolute or a relative path
+                let prefix_path = path_to_config.parent().unwrap().parent().unwrap();
+                // Joining an absolute path to any path results in the given absolute path
+                Ok(Some(prefix_path.join(specified_path)))
+            } else {
+                // A pathless name
+                Ok(Some(PathBuf::from(specified_path)))
+            }
+        } else {
+            Ok(None)
+        }
+    }
+
     pub fn get_list(&self, key: &str) -> CargoResult<Option<(Vec<(String, PathBuf)>, PathBuf)>> {
         match try!(self.get(key)) {
             Some(CV::List(i, path)) => Ok(Some((i, path))),
@@ -219,9 +231,7 @@ impl Config {
     }
 
     fn scrape_rustc_version(&mut self) -> CargoResult<()> {
-        let (rustc_version, rustc_host) = try!(ops::rustc_version(&self.rustc));
-        self.rustc_version = rustc_version;
-        self.rustc_host = rustc_host;
+        self.rustc_info = try!(Rustc::new(&self.rustc));
         Ok(())
     }
 
@@ -231,25 +241,17 @@ impl Config {
             path.pop();
             path.pop();
             path.push(dir);
-            self.target_dir = Some(path);
+            *self.target_dir.borrow_mut() = Some(path);
         } else if let Some(dir) = env::var_os("CARGO_TARGET_DIR") {
-            self.target_dir = Some(self.cwd.join(dir));
+            *self.target_dir.borrow_mut() = Some(self.cwd.join(dir));
         }
         Ok(())
     }
 
     fn get_tool(&self, tool: &str) -> CargoResult<PathBuf> {
         let var = format!("build.{}", tool);
-        if let Some((tool, path)) = try!(self.get_string(&var)) {
-            // If this tool has a path separator in it, calculate the path
-            // relative to the config file (specified by `path`). To do that we
-            // chop off the `.cargo/config` at the end of the path and then join
-            // on the tool.
-            if tool.contains("/") || (cfg!(windows) && tool.contains("\\")) {
-                let path = path.parent().unwrap().parent().unwrap();
-                return Ok(path.join(tool))
-            }
-            return Ok(PathBuf::from(tool))
+        if let Some(tool_path) = try!(self.get_path(&var)) {
+            return Ok(tool_path);
         }
 
         let var = tool.chars().flat_map(|c| c.to_uppercase()).collect::<String>();
@@ -511,11 +513,11 @@ pub fn set_config(cfg: &Config, loc: Location, key: &str,
         Location::Project => unimplemented!(),
     };
     try!(fs::create_dir_all(file.parent().unwrap()));
-    let mut contents = String::new();
-    let _ = File::open(&file).and_then(|mut f| f.read_to_string(&mut contents));
+    let contents = paths::read(&file).unwrap_or(String::new());
     let mut toml = try!(cargo_toml::parse(&contents, &file));
     toml.insert(key.to_string(), value.into_toml());
-    let mut out = try!(File::create(&file));
-    try!(out.write_all(toml::Value::Table(toml).to_string().as_bytes()));
+
+    let contents = toml::Value::Table(toml).to_string();
+    try!(paths::write(&file, contents.as_bytes()));
     Ok(())
 }
