@@ -55,7 +55,6 @@ impl<'a, T, F> ChainError<T> for F where F: FnOnce() -> CargoResult<T> {
 }
 
 impl<T, E: CargoError + 'static> ChainError<T> for Result<T, E> {
-    #[allow(trivial_casts)]
     fn chain_error<E2: 'static, C>(self, callback: C) -> CargoResult<T>
                          where E2: CargoError, C: FnOnce() -> E2 {
         self.map_err(move |err| {
@@ -114,7 +113,6 @@ pub struct ProcessError {
 
 impl Error for ProcessError {
     fn description(&self) -> &str { &self.desc }
-    #[allow(trivial_casts)]
     fn cause(&self) -> Option<&Error> {
         self.cause.as_ref().map(|s| s as &Error)
     }
@@ -131,6 +129,53 @@ impl fmt::Debug for ProcessError {
     }
 }
 
+// =============================================================================
+// Cargo test errors.
+
+/// Error when testcases fail
+pub struct CargoTestError {
+    pub desc: String,
+    pub exit: Option<ExitStatus>,
+    pub causes: Vec<ProcessError>,
+}
+
+impl CargoTestError {
+    #[allow(deprecated)] // connect => join in 1.3
+    pub fn new(errors: Vec<ProcessError>) -> Self {
+        if errors.len() == 0 {
+            panic!("Cannot create CargoTestError from empty Vec")
+        }
+        let desc = errors.iter().map(|error| error.desc.clone())
+                                .collect::<Vec<String>>()
+                                .connect("\n");
+        CargoTestError {
+            desc: desc,
+            exit: errors[0].exit,
+            causes: errors,
+        }
+    }
+}
+
+impl fmt::Display for CargoTestError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        fmt::Display::fmt(&self.desc, f)
+    }
+}
+
+impl fmt::Debug for CargoTestError {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        fmt::Display::fmt(self, f)
+    }
+}
+
+impl Error for CargoTestError {
+    fn description(&self) -> &str { &self.desc }
+    fn cause(&self) -> Option<&Error> {
+        self.causes.get(0).map(|s| s as &Error)
+    }
+}
+
+
 // =============================================================================
 // Concrete errors
 
@@ -274,6 +319,7 @@ impl CargoError for git2::Error {}
 impl CargoError for json::DecoderError {}
 impl CargoError for curl::ErrCode {}
 impl CargoError for ProcessError {}
+impl CargoError for CargoTestError {}
 impl CargoError for CliError {}
 impl CargoError for toml::Error {}
 impl CargoError for toml::DecodeError {}
