@@ -7,7 +7,7 @@ use tempdir::TempDir;
 use support::{project, execs, main_file, basic_bin_manifest};
 use support::{COMPILING, RUNNING, ProjectBuilder};
 use hamcrest::{assert_that, existing_file, is_not};
-use support::paths::CargoPathExt;
+use support::paths::{CargoPathExt,root};
 use cargo::util::process;
 
 fn setup() {
@@ -650,7 +650,30 @@ consider running `cargo update` to update a path dependency's locked version
 "));
 });
 
-// test!(compiling_project_with_invalid_manifest)
+test!(ignores_carriage_return_in_lockfile {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            authors = []
+            version = "0.0.1"
+        "#)
+        .file("src/main.rs", r#"
+            mod a; fn main() {}
+        "#)
+        .file("src/a.rs", "");
+
+    assert_that(p.cargo_process("build"),
+                execs().with_status(0));
+
+    let lockfile = p.root().join("Cargo.lock");
+    let mut lock = String::new();
+    File::open(&lockfile).unwrap().read_to_string(&mut lock).unwrap();
+    let lock = lock.replace("\n", "\r\n");
+    File::create(&lockfile).unwrap().write_all(lock.as_bytes()).unwrap();
+    assert_that(p.cargo("build"),
+                execs().with_status(0));
+});
 
 test!(crate_version_env_vars {
     let p = project("foo")
@@ -722,24 +745,12 @@ test!(many_crate_types_old_style_lib_location {
         .file("src/foo.rs", r#"
             pub fn foo() {}
         "#);
-    assert_that(p.cargo_process("build"),
-                execs().with_status(0));
+    assert_that(p.cargo_process("build"), execs().with_status(0));
 
-    let files = fs::read_dir(&p.root().join("target/debug")).unwrap();
-    let mut files: Vec<String> = files.map(|e| e.unwrap().path()).filter_map(|f| {
-        match f.file_name().unwrap().to_str().unwrap() {
-            "build" | "examples" | "deps" => None,
-            s if s.contains("fingerprint") || s.contains("dSYM") => None,
-            s => Some(s.to_string())
-        }
-    }).collect();
-    files.sort();
-    let file0 = &files[0];
-    let file1 = &files[1];
-    println!("{} {}", file0, file1);
-    assert!(file0.ends_with(".rlib") || file1.ends_with(".rlib"));
-    assert!(file0.ends_with(env::consts::DLL_SUFFIX) ||
-            file1.ends_with(env::consts::DLL_SUFFIX));
+    assert_that(&p.root().join("target/debug/libfoo.rlib"), existing_file());
+    let fname = format!("{}foo{}", env::consts::DLL_PREFIX,
+                        env::consts::DLL_SUFFIX);
+    assert_that(&p.root().join("target/debug").join(&fname), existing_file());
 });
 
 test!(many_crate_types_correct {
@@ -763,21 +774,10 @@ test!(many_crate_types_correct {
     assert_that(p.cargo_process("build"),
                 execs().with_status(0));
 
-    let files = fs::read_dir(&p.root().join("target/debug")).unwrap();
-    let mut files: Vec<String> = files.map(|f| f.unwrap().path()).filter_map(|f| {
-        match f.file_name().unwrap().to_str().unwrap() {
-            "build" | "examples" | "deps" => None,
-            s if s.contains("fingerprint") || s.contains("dSYM") => None,
-            s => Some(s.to_string())
-        }
-    }).collect();
-    files.sort();
-    let file0 = &files[0];
-    let file1 = &files[1];
-    println!("{} {}", file0, file1);
-    assert!(file0.ends_with(".rlib") || file1.ends_with(".rlib"));
-    assert!(file0.ends_with(env::consts::DLL_SUFFIX) ||
-            file1.ends_with(env::consts::DLL_SUFFIX));
+    assert_that(&p.root().join("target/debug/libfoo.rlib"), existing_file());
+    let fname = format!("{}foo{}", env::consts::DLL_PREFIX,
+                        env::consts::DLL_SUFFIX);
+    assert_that(&p.root().join("target/debug").join(&fname), existing_file());
 });
 
 test!(unused_keys {
@@ -887,6 +887,11 @@ Caused by:
 });
 
 test!(lto_build {
+    // FIXME: currently this hits a linker bug on 32-bit MSVC
+    if cfg!(all(target_env = "msvc", target_pointer_width = "32")) {
+        return
+    }
+
     let mut p = project("foo");
     p = p
         .file("Cargo.toml", r#"
@@ -1526,7 +1531,7 @@ test!(cargo_platform_specific_dependency {
                 execs().with_status(0));
 
     assert_that(&p.bin("foo"), existing_file());
-    assert_that(p.cargo_process("test"),
+    assert_that(p.cargo("test"),
                 execs().with_status(0));
 });
 
@@ -1681,7 +1686,7 @@ test!(transitive_dependencies_not_available {
     assert_that(p.cargo_process("build").arg("-v"),
                 execs().with_status(101)
                        .with_stderr("\
-[..] can't find crate for `bbbbb`
+[..] can't find crate for `bbbbb`[..]
 [..] extern crate bbbbb; [..]
 [..]
 error: aborting due to previous error
@@ -1798,7 +1803,8 @@ test!(rustc_env_var {
 Could not execute process `rustc-that-does-not-exist -vV` ([..])
 
 Caused by:
-[..]".to_string() + if cfg!(windows) {"\n[..]\n"} else {"\n"}));
+[..]
+"));
     assert_that(&p.bin("a"), is_not(existing_file()));
 });
 
@@ -1845,6 +1851,38 @@ test!(ignore_dotfile {
                 execs().with_status(0));
 });
 
+test!(ignore_dotdirs {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+        "#)
+        .file("src/bin/a.rs", "fn main() {}")
+        .file(".git/Cargo.toml", "")
+        .file(".pc/dummy-fix.patch/Cargo.toml", "");
+    p.build();
+
+    assert_that(p.cargo("build"),
+                execs().with_status(0));
+});
+
+test!(dotdir_root {
+    let p = ProjectBuilder::new("foo", root().join(".foo"))
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+        "#)
+        .file("src/bin/a.rs", "fn main() {}");
+    p.build();
+    assert_that(p.cargo("build"),
+                execs().with_status(0));
+});
+
+
 test!(custom_target_dir {
     let p = project("foo")
         .file("Cargo.toml", r#"
@@ -1901,3 +1939,116 @@ test!(rustc_no_trans {
     assert_that(p.cargo("rustc").arg("-v").arg("--").arg("-Zno-trans"),
                 execs().with_status(0));
 });
+
+test!(build_multiple_packages {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+
+            [dependencies.d1]
+                path = "d1"
+            [dependencies.d2]
+                path = "d2"
+
+            [[bin]]
+                name = "foo"
+        "#)
+        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
+        .file("d1/Cargo.toml", r#"
+            [package]
+            name = "d1"
+            version = "0.0.1"
+            authors = []
+
+            [[bin]]
+                name = "d1"
+        "#)
+        .file("d1/src/lib.rs", "")
+        .file("d1/src/main.rs", "fn main() { println!(\"d1\"); }")
+        .file("d2/Cargo.toml", r#"
+            [package]
+            name = "d2"
+            version = "0.0.1"
+            authors = []
+
+            [[bin]]
+                name = "d2"
+                doctest = false
+        "#)
+        .file("d2/src/main.rs", "fn main() { println!(\"d2\"); }");
+    p.build();
+
+    assert_that(p.cargo_process("build").arg("-p").arg("d1").arg("-p").arg("d2")
+                                        .arg("-p").arg("foo"),
+                execs());
+
+    assert_that(&p.bin("foo"), existing_file());
+    assert_that(process(&p.bin("foo")).unwrap(),
+                execs().with_stdout("i am foo\n"));
+
+    let d1_path = &p.build_dir().join("debug").join("deps")
+                                .join(format!("d1{}", env::consts::EXE_SUFFIX));
+    let d2_path = &p.build_dir().join("debug").join("deps")
+                                .join(format!("d2{}", env::consts::EXE_SUFFIX));
+
+    assert_that(d1_path, existing_file());
+    assert_that(process(d1_path).unwrap(), execs().with_stdout("d1"));
+
+    assert_that(d2_path, existing_file());
+    assert_that(process(d2_path).unwrap(),
+                execs().with_stdout("d2"));
+});
+
+test!(invalid_spec {
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+
+            [dependencies.d1]
+                path = "d1"
+
+            [[bin]]
+                name = "foo"
+        "#)
+        .file("src/foo.rs", &main_file(r#""i am foo""#, &[]))
+        .file("d1/Cargo.toml", r#"
+            [package]
+            name = "d1"
+            version = "0.0.1"
+            authors = []
+
+            [[bin]]
+                name = "d1"
+        "#)
+        .file("d1/src/lib.rs", "")
+        .file("d1/src/main.rs", "fn main() { println!(\"d1\"); }");
+    p.build();
+
+    assert_that(p.cargo_process("build").arg("-p").arg("notAValidDep"),
+                execs().with_status(101).with_stderr(
+                    "could not find package matching spec `notAValidDep`".to_string()));
+
+    assert_that(p.cargo_process("build").arg("-p").arg("d1").arg("-p").arg("notAValidDep"),
+                execs().with_status(101).with_stderr(
+                    "could not find package matching spec `notAValidDep`".to_string()));
+
+});
+
+test!(manifest_with_bom_is_ok {
+    let p = project("foo")
+        .file("Cargo.toml", "\u{FEFF}
+            [package]
+            name = \"foo\"
+            version = \"0.0.1\"
+            authors = []
+        ")
+        .file("src/lib.rs", "");
+    assert_that(p.cargo_process("build").arg("-v"),
+                execs().with_status(0));
+});
