@@ -18,13 +18,30 @@ fn disabled() -> bool {
     // Right now the windows bots cannot cross compile due to the mingw setup,
     // so we disable ourselves on all but macos/linux setups where the rustc
     // install script ensures we have both architectures
-    return !cfg!(target_os = "macos") && !cfg!(target_os = "linux");
+    !(cfg!(target_os = "macos") ||
+      cfg!(target_os = "linux") ||
+      cfg!(target_env = "msvc"))
 }
 
-fn alternate() -> &'static str {
-    match env::consts::OS {
-        "linux" => "i686-unknown-linux-gnu",
-        "macos" => "i686-apple-darwin",
+fn alternate() -> String {
+    let platform = match env::consts::OS {
+        "linux" => "unknown-linux-gnu",
+        "macos" => "apple-darwin",
+        "windows" => "pc-windows-msvc",
+        _ => unreachable!(),
+    };
+    let arch = match env::consts::ARCH {
+        "x86" => "x86_64",
+        "x86_64" => "i686",
+        _ => unreachable!(),
+    };
+    format!("{}-{}", arch, platform)
+}
+
+fn alternate_arch() -> &'static str {
+    match env::consts::ARCH {
+        "x86" => "x86_64",
+        "x86_64" => "x86",
         _ => unreachable!(),
     }
 }
@@ -45,19 +62,19 @@ test!(simple_cross {
                 assert_eq!(std::env::var("TARGET").unwrap(), "{}");
             }}
         "#, alternate()))
-        .file("src/main.rs", r#"
+        .file("src/main.rs", &format!(r#"
             use std::env;
-            fn main() {
-                assert_eq!(env::consts::ARCH, "x86");
-            }
-        "#);
+            fn main() {{
+                assert_eq!(env::consts::ARCH, "{}");
+            }}
+        "#, alternate_arch()));
 
     let target = alternate();
-    assert_that(p.cargo_process("build").arg("--target").arg(target),
+    assert_that(p.cargo_process("build").arg("--target").arg(&target).arg("-v"),
                 execs().with_status(0));
-    assert_that(&p.target_bin(target, "foo"), existing_file());
+    assert_that(&p.target_bin(&target, "foo"), existing_file());
 
-    assert_that(process(&p.target_bin(target, "foo")).unwrap(),
+    assert_that(process(&p.target_bin(&target, "foo")).unwrap(),
                 execs().with_status(0));
 });
 
@@ -89,11 +106,11 @@ test!(simple_deps {
     p2.build();
 
     let target = alternate();
-    assert_that(p.cargo_process("build").arg("--target").arg(target),
+    assert_that(p.cargo_process("build").arg("--target").arg(&target),
                 execs().with_status(0));
-    assert_that(&p.target_bin(target, "foo"), existing_file());
+    assert_that(&p.target_bin(&target, "foo"), existing_file());
 
-    assert_that(process(&p.target_bin(target, "foo")).unwrap(),
+    assert_that(process(&p.target_bin(&target, "foo")).unwrap(),
                 execs().with_status(0));
 });
 
@@ -166,11 +183,11 @@ test!(plugin_deps {
     baz.build();
 
     let target = alternate();
-    assert_that(foo.cargo_process("build").arg("--target").arg(target),
+    assert_that(foo.cargo_process("build").arg("--target").arg(&target),
                 execs().with_status(0));
-    assert_that(&foo.target_bin(target, "foo"), existing_file());
+    assert_that(&foo.target_bin(&target, "foo"), existing_file());
 
-    assert_that(process(&foo.target_bin(target, "foo")).unwrap(),
+    assert_that(process(&foo.target_bin(&target, "foo")).unwrap(),
                 execs().with_status(0));
 });
 
@@ -247,15 +264,15 @@ test!(plugin_to_the_max {
     baz.build();
 
     let target = alternate();
-    assert_that(foo.cargo_process("build").arg("--target").arg(target).arg("-v"),
+    assert_that(foo.cargo_process("build").arg("--target").arg(&target).arg("-v"),
                 execs().with_status(0));
     println!("second");
     assert_that(foo.cargo("build").arg("-v")
-                   .arg("--target").arg(target),
+                   .arg("--target").arg(&target),
                 execs().with_status(0));
-    assert_that(&foo.target_bin(target, "foo"), existing_file());
+    assert_that(&foo.target_bin(&target, "foo"), existing_file());
 
-    assert_that(process(&foo.target_bin(target, "foo")).unwrap(),
+    assert_that(process(&foo.target_bin(&target, "foo")).unwrap(),
                 execs().with_status(0));
 });
 
@@ -270,19 +287,19 @@ test!(linker_and_ar {
             linker = "my-linker-tool"
         "#, target))
         .file("Cargo.toml", &basic_bin_manifest("foo"))
-        .file("src/foo.rs", r#"
+        .file("src/foo.rs", &format!(r#"
             use std::env;
-            fn main() {
-                assert_eq!(env::consts::ARCH, "x86");
-            }
-        "#);
+            fn main() {{
+                assert_eq!(env::consts::ARCH, "{}");
+            }}
+        "#, alternate_arch()));
 
-    assert_that(p.cargo_process("build").arg("--target").arg(target)
+    assert_that(p.cargo_process("build").arg("--target").arg(&target)
                                               .arg("-v"),
                 execs().with_status(101)
                        .with_stdout(&format!("\
 {compiling} foo v0.5.0 ({url})
-{running} `rustc src/foo.rs --crate-name foo --crate-type bin -g \
+{running} `rustc src[..]foo.rs --crate-name foo --crate-type bin -g \
     --out-dir {dir}[..]target[..]{target}[..]debug \
     --emit=dep-info,link \
     --target {target} \
@@ -361,7 +378,7 @@ test!(plugin_with_extra_dylib_dep {
     baz.build();
 
     let target = alternate();
-    assert_that(foo.cargo_process("build").arg("--target").arg(target),
+    assert_that(foo.cargo_process("build").arg("--target").arg(&target),
                 execs().with_status(0));
 });
 
@@ -378,22 +395,22 @@ test!(cross_tests {
             [[bin]]
             name = "bar"
         "#)
-        .file("src/main.rs", r#"
+        .file("src/main.rs", &format!(r#"
             extern crate foo;
             use std::env;
-            fn main() {
-                assert_eq!(env::consts::ARCH, "x86");
-            }
-            #[test] fn test() { main() }
-        "#)
-        .file("src/lib.rs", r#"
+            fn main() {{
+                assert_eq!(env::consts::ARCH, "{}");
+            }}
+            #[test] fn test() {{ main() }}
+        "#, alternate_arch()))
+        .file("src/lib.rs", &format!(r#"
             use std::env;
-            pub fn foo() { assert_eq!(env::consts::ARCH, "x86"); }
-            #[test] fn test_foo() { foo() }
-        "#);
+            pub fn foo() {{ assert_eq!(env::consts::ARCH, "{}"); }}
+            #[test] fn test_foo() {{ foo() }}
+        "#, alternate_arch()));
 
     let target = alternate();
-    assert_that(p.cargo_process("test").arg("--target").arg(target),
+    assert_that(p.cargo_process("test").arg("--target").arg(&target),
                 execs().with_status(0)
                        .with_stdout(&format!("\
 {compiling} foo v0.0.0 ({foo})
@@ -431,15 +448,15 @@ test!(simple_cargo_run {
             version = "0.0.0"
             authors = []
         "#)
-        .file("src/main.rs", r#"
+        .file("src/main.rs", &format!(r#"
             use std::env;
-            fn main() {
-                assert_eq!(env::consts::ARCH, "x86");
-            }
-        "#);
+            fn main() {{
+                assert_eq!(env::consts::ARCH, "{}");
+            }}
+        "#, alternate_arch()));
 
     let target = alternate();
-    assert_that(p.cargo_process("run").arg("--target").arg(target),
+    assert_that(p.cargo_process("run").arg("--target").arg(&target),
                 execs().with_status(0));
 });
 
@@ -548,26 +565,29 @@ test!(build_script_needed_for_host_and_target {
 
     assert_that(p.cargo_process("build").arg("--target").arg(&target).arg("-v"),
                 execs().with_status(0)
-                       .with_stdout(&format!("\
-{compiling} d1 v0.0.0 (file://{dir})
-{running} `rustc d1[..]build.rs [..] --out-dir {dir}[..]target[..]build[..]d1-[..]`
-{running} `{dir}[..]target[..]build[..]d1-[..]build-script-build`
-{running} `{dir}[..]target[..]build[..]d1-[..]build-script-build`
-{running} `rustc d1[..]src[..]lib.rs [..] --target {target} [..] \
-           -L /path/to/{target}`
-{running} `rustc d1[..]src[..]lib.rs [..] \
-           -L /path/to/{host}`
-{compiling} d2 v0.0.0 (file://{dir})
+                       .with_stdout_contains(&format!("\
+{compiling} d1 v0.0.0 ({url})", compiling = COMPILING, url = p.url()))
+                       .with_stdout_contains(&format!("\
+{running} `rustc d1[..]build.rs [..] --out-dir {dir}[..]target[..]build[..]d1-[..]`",
+    running = RUNNING, dir = p.root().display()))
+                       .with_stdout_contains(&format!("\
+{running} `{dir}[..]target[..]build[..]d1-[..]build-script-build`", running = RUNNING,
+    dir = p.root().display()))
+                       .with_stdout_contains(&format!("\
+{running} `rustc d1[..]src[..]lib.rs [..]`", running = RUNNING))
+                       .with_stdout_contains(&format!("\
+{compiling} d2 v0.0.0 ({url})", compiling = COMPILING, url = p.url()))
+                       .with_stdout_contains(&format!("\
 {running} `rustc d2[..]src[..]lib.rs [..] \
-           -L /path/to/{host}`
-{compiling} foo v0.0.0 (file://{dir})
+           -L /path/to/{host}`", running = RUNNING, host = host))
+                       .with_stdout_contains(&format!("\
+{compiling} foo v0.0.0 ({url})", compiling = COMPILING, url = p.url()))
+                       .with_stdout_contains(&format!("\
 {running} `rustc build.rs [..] --out-dir {dir}[..]target[..]build[..]foo-[..] \
-           -L /path/to/{host}`
-{running} `{dir}[..]target[..]build[..]foo-[..]build-script-build`
+           -L /path/to/{host}`", running = RUNNING, dir = p.root().display(), host = host))
+                       .with_stdout_contains(&format!("\
 {running} `rustc src[..]main.rs [..] --target {target} [..] \
-           -L /path/to/{target}`
-", compiling = COMPILING, running = RUNNING, target = target, host = host,
-   dir = p.root().display())));
+           -L /path/to/{target}`", running = RUNNING, target = target)));
 });
 
 test!(build_deps_for_the_right_arch {
@@ -641,8 +661,8 @@ test!(build_script_only_host {
             use std::env;
 
             fn main() {
-                assert!(env::var("OUT_DIR").unwrap()
-                                             .contains("target/debug/build/d1-"),
+                assert!(env::var("OUT_DIR").unwrap().replace("\\", "/")
+                                           .contains("target/debug/build/d1-"),
                         "bad: {:?}", env::var("OUT_DIR"));
             }
         "#);
@@ -678,3 +698,173 @@ test!(plugin_build_script_right_arch {
 {running} `rustc src[..]lib.rs [..]`
 ", compiling = COMPILING, running = RUNNING)));
 });
+
+test!(build_script_with_platform_specific_dependencies {
+    if disabled() { return }
+
+    let target = alternate();
+    let host = ::rustc_host();
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            build = "build.rs"
+
+            [build-dependencies.d1]
+            path = "d1"
+        "#)
+        .file("build.rs", "extern crate d1; fn main() {}")
+        .file("src/lib.rs", "")
+        .file("d1/Cargo.toml", &format!(r#"
+            [package]
+            name = "d1"
+            version = "0.0.0"
+            authors = []
+
+            [target.{}.dependencies]
+            d2 = {{ path = "../d2" }}
+        "#, host))
+        .file("d1/src/lib.rs", "extern crate d2;")
+        .file("d2/Cargo.toml", r#"
+            [package]
+            name = "d2"
+            version = "0.0.0"
+            authors = []
+        "#)
+        .file("d2/src/lib.rs", "");
+
+    assert_that(p.cargo_process("build").arg("-v").arg("--target").arg(&target),
+                execs().with_status(0)
+                       .with_stdout(&format!("\
+{compiling} d2 v0.0.0 ([..])
+{running} `rustc d2[..]src[..]lib.rs [..]`
+{compiling} d1 v0.0.0 ([..])
+{running} `rustc d1[..]src[..]lib.rs [..]`
+{compiling} foo v0.0.1 ([..])
+{running} `rustc build.rs [..]`
+{running} `{dir}[..]target[..]build[..]foo-[..]build-script-build`
+{running} `rustc src[..]lib.rs [..] --target {target} [..]`
+", compiling = COMPILING, running = RUNNING, dir = p.root().display(), target = target)));
+});
+
+test!(platform_specific_dependencies_do_not_leak {
+    if disabled() { return }
+
+    let target = alternate();
+    let host = ::rustc_host();
+    let p = project("foo")
+        .file("Cargo.toml", r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            build = "build.rs"
+
+            [dependencies.d1]
+            path = "d1"
+
+            [build-dependencies.d1]
+            path = "d1"
+        "#)
+        .file("build.rs", "extern crate d1; fn main() {}")
+        .file("src/lib.rs", "")
+        .file("d1/Cargo.toml", &format!(r#"
+            [package]
+            name = "d1"
+            version = "0.0.0"
+            authors = []
+
+            [target.{}.dependencies]
+            d2 = {{ path = "../d2" }}
+        "#, host))
+        .file("d1/src/lib.rs", "extern crate d2;")
+        .file("d2/Cargo.toml", r#"
+            [package]
+            name = "d2"
+            version = "0.0.0"
+            authors = []
+        "#)
+        .file("d2/src/lib.rs", "");
+
+    assert_that(p.cargo_process("build").arg("-v").arg("--target").arg(&target),
+                execs().with_status(101)
+                       .with_stderr("\
+[..] error: can't find crate for `d2`[..]
+[..] extern crate d2;
+[..]
+error: aborting due to previous error
+Could not compile `d1`.
+
+Caused by:
+  [..]
+"));
+});
+
+test!(platform_specific_variables_reflected_in_build_scripts {
+    if disabled() { return }
+
+    let target = alternate();
+    let host = ::rustc_host();
+    let p = project("foo")
+        .file("Cargo.toml", &format!(r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            build = "build.rs"
+
+            [target.{host}.dependencies]
+            d1 = {{ path = "d1" }}
+
+            [target.{target}.dependencies]
+            d2 = {{ path = "d2" }}
+        "#, host = host, target = target))
+        .file("build.rs", &format!(r#"
+            use std::env;
+
+            fn main() {{
+                let platform = env::var("TARGET").unwrap();
+                let (expected, not_expected) = match &platform[..] {{
+                    "{host}" => ("DEP_D1_VAL", "DEP_D2_VAL"),
+                    "{target}" => ("DEP_D2_VAL", "DEP_D1_VAL"),
+                    _ => panic!("unknown platform")
+                }};
+
+                env::var(expected).ok()
+                    .expect(&format!("missing {{}}", expected));
+                env::var(not_expected).err()
+                    .expect(&format!("found {{}}", not_expected));
+            }}
+        "#, host = host, target = target))
+        .file("src/lib.rs", "")
+        .file("d1/Cargo.toml", r#"
+            [package]
+            name = "d1"
+            version = "0.0.0"
+            authors = []
+            links = "d1"
+            build = "build.rs"
+        "#)
+        .file("d1/build.rs", r#"
+            fn main() { println!("cargo:val=1") }
+        "#)
+        .file("d1/src/lib.rs", "")
+        .file("d2/Cargo.toml", r#"
+            [package]
+            name = "d2"
+            version = "0.0.0"
+            authors = []
+            links = "d2"
+            build = "build.rs"
+        "#)
+        .file("d2/build.rs", r#"
+            fn main() { println!("cargo:val=1") }
+        "#)
+        .file("d2/src/lib.rs", "");
+
+    assert_that(p.cargo_process("build").arg("-v"), execs().with_status(0));
+    assert_that(p.cargo_process("build").arg("-v").arg("--target").arg(&target),
+                execs().with_status(0));
+});
