@@ -16,6 +16,8 @@ extern crate url;
 #[macro_use]
 extern crate log;
 
+use cargo::util::Rustc;
+
 mod support;
 macro_rules! test {
     ($name:ident $expr:expr) => (
@@ -29,6 +31,7 @@ macro_rules! test {
 }
 
 mod test_bad_config;
+mod test_bad_manifest_path;
 mod test_cargo;
 mod test_cargo_bench;
 mod test_cargo_build_auth;
@@ -45,23 +48,35 @@ mod test_cargo_features;
 mod test_cargo_fetch;
 mod test_cargo_freshness;
 mod test_cargo_generate_lockfile;
+mod test_cargo_install;
 mod test_cargo_new;
 mod test_cargo_package;
 mod test_cargo_profiles;
 mod test_cargo_publish;
+mod test_cargo_read_manifest;
 mod test_cargo_registry;
 mod test_cargo_run;
 mod test_cargo_rustc;
 mod test_cargo_search;
 mod test_cargo_test;
+mod test_cargo_tool_paths;
+mod test_cargo_verify_project;
 mod test_cargo_version;
 mod test_shell;
 
+thread_local!(static RUSTC: Rustc = Rustc::new("rustc").unwrap());
+
 fn rustc_host() -> String {
-    cargo::ops::rustc_version("rustc").unwrap().1
+    RUSTC.with(|r| r.host.clone())
 }
 
 fn is_nightly() -> bool {
-    let version_info = cargo::ops::rustc_version("rustc").unwrap().0;
-    version_info.contains("-nightly") || version_info.contains("-dev")
+    RUSTC.with(|r| {
+        r.verbose_version.contains("-nightly") ||
+            r.verbose_version.contains("-dev")
+    })
+}
+
+fn can_panic() -> bool {
+    RUSTC.with(|r| !(r.host.contains("msvc") && !r.host.contains("x86_64")))
 }
